
def recurse(root, filename, nodes):    
    
    for child in root.GetContent():        
        obj = JsonObject_Item(filename, root, child)        
        nodes.append(obj)        

    for child in root.GetSubHeadings():
        obj = JsonObject_Item(filename, root, child)        
        nodes.append(obj)
        recurse(child, filename, nodes)

def ToString(roots):
    nodes = []
    for root in roots: 
        obj = JsonObject_Item(root.Content, root, root)
        obj["Parent"] = ''
        nodes.append(obj)
        recurse(root, root.Content, nodes)
    return nodes

def JsonObject_Item(filename, parent, child):

    obj = {"document": filename, "Parent": parent.GUID, "GUID": child.GUID, "Content": "", "TableContent": [], "ColumnNames": [],  "Type": child.Type, "Tags": [], "QA":[]}
        
    if (child.Type is "Table"):
        df = child.Content
        obj["ColumnNames"] = list(df.columns.values)
        Row_list =[]   
        # Iterate over each row 
        for i in range((df.shape[0])):
            # Using iloc to access the values of  
            # the current row denoted by "i" 
            Row_list.append(list(df.iloc[i, :]))  
        obj["TableContent"] = Row_list
        pass
    else:
        obj["Content"] = child.Content

    if hasattr(child, 'Tags'):
        obj["Tags"] = child.Tags
    if hasattr(child, 'QA'):
        obj["QA"] = child.QA

    return obj


def GetAllHeadings(item, filename):
    json = []
    for head in item.GetSubHeadings():
        obj = JsonObject_Item(filename,item,head)        
        items = GetAllHeadings(head, filename)
        obj["Items"] = items
        json.append(obj)
    return json
    

def ToString_AllHeadings(item):
    data = GetAllHeadings(item, item.Content)
    return data


import json
from json import JSONEncoder
import uuid 


class Item(JSONEncoder):

    def __init__(self):
        self.GUID = str(uuid.uuid4())
        self.Type = ''        
        self.Content = ''
        self.N = None
        self.Parent = None
        self.Items = []
    
    
    def GetSubHeadings(self):
        heads = list(filter(lambda item: 'Heading' in item.Type, self.Items))
        items = []
        for index, head in enumerate(heads):
            if index == 0:                
                continue    # reason we're ignoring the first element because it is redundant heading i.e (head == subhead)
            items.append(head)
        return items   
    
    def GetTables(self):
        return list(filter(lambda item: 'Table' in item.Type, self.Items))

    def GetContent(self):
        return list(filter(lambda item: 'Heading' not in item.Type, self.Items))
    
    def GetChildren(self):
        return self.Items
    
    def GetChild(self, guid):
        return next((item for item in self.Items if item.GUID == guid), None)
    
    def toJSON(self):
        return json.dumps(self, default=lambda obj: obj.__dict__)

    def Find(self, guid):
        return self.FindItem(guid)

    def FindItem(self, guid):
        obj = None
        for head in self.GetSubHeadings(): 
            if obj is None:             
                if str(head.GUID).strip() == str(guid).strip():                     
                    obj = head                    
                else:
                    obj = head.FindItem(guid)
                
        
        return obj








import os
import re
from setuptools import find_packages, setup


def text_of(relpath):
    """
    Return string containing the contents of the file at *relpath* relative to
    this file.
    """
    thisdir = os.path.dirname(__file__)
    file_path = os.path.join(thisdir, os.path.normpath(relpath))
    with open(file_path, encoding='utf-8') as f:
        text = f.read()
    return text

# Read the version from docx.__version__ without importing the package
# (and thus attempting to import packages it depends on that may not be
# installed yet)
version = re.search(
    "__version__ = '([^']+)'", text_of('mswordtree/__init__.py')
).group(1)


NAME = 'mswordtree'
VERSION = version
DESCRIPTION = 'Get the parsed microsoft word document in a hierarchical tree structure.'
KEYWORDS = 'docx office openxml word tree microsoft headings tables'
AUTHOR = 'Ali Asad'
AUTHOR_EMAIL = 'imaliasad@outlook.com'
URL = 'https://github.com/imAliAsad/mswordtree'
LICENSE = 'MIT License' #text_of('LICENSE')
PACKAGES = find_packages()
INSTALL_REQUIRES = ['pandas', 'python-docx', 'uuid']
LONG_DESCRIPTION = text_of('README.md')
LONG_DESCRIPTION_Content_Type = 'text/markdown'

CLASSIFIERS = [ 'Development Status :: 3 - Alpha', 'Environment :: Console', 'Intended Audience :: Developers', 'License :: OSI Approved :: MIT License', 'Operating System :: OS Independent', 'Programming Language :: Python', 'Programming Language :: Python :: 2', 'Programming Language :: Python :: 2.6', 'Programming Language :: Python :: 2.7', 'Programming Language :: Python :: 3', 'Programming Language :: Python :: 3.3', 'Programming Language :: Python :: 3.4', 'Topic :: Office/Business :: Office Suites', 'Topic :: Software Development :: Libraries' ]
params = {
    'name':             NAME,
    'version':          VERSION,
    'description':      DESCRIPTION,
    'keywords':         KEYWORDS,
    'long_description': LONG_DESCRIPTION,
    'long_description_content_type': LONG_DESCRIPTION_Content_Type,
    'author':           AUTHOR,
    'author_email':     AUTHOR_EMAIL,
    'url':              URL,
    'license':          LICENSE,
    'packages':         PACKAGES,    
    'install_requires': INSTALL_REQUIRES,
    'classifiers':      CLASSIFIERS,
}


setup(**params)


from docx import Document
from docx.document import Document as _Document
from docx.oxml.text.paragraph import CT_P
from docx.oxml.table import CT_Tbl
from docx.table import _Cell, Table
from docx.text.paragraph import Paragraph
import pandas as pd
from mswordtree.Item import Item

def iter_block_items(parent):
    if isinstance(parent, _Document):
        parent_elm = parent.element.body
        
    elif isinstance(parent, _Cell):
        parent_elm = parent._tc
    else:
        raise ValueError("something's not right")

    for child in parent_elm.iterchildren():
        if isinstance(child, CT_P):
            yield Paragraph(child, parent)
        elif isinstance(child, CT_Tbl):
            yield Table(child, parent)
            
def ParseTableToDataFrame(table):
    data = []
    keys = None
    
    gtext = [] # horizontal row text
    isVertical = True
    for i, row in enumerate(table.rows):
        text = list((cell.text for cell in row.cells))
        if i == 0:
            # if its a vertical table, its heading will have bold property
            try:
                isVertical = all([cell.paragraphs[0].runs[0].font.bold for cell in row.cells])
            except:
                isVertical = True
                
            if isVertical == True:            
                keys = tuple(text)                
                continue
            # else its a horizontal table
            else:  
                keys = [key for key in ((cell.text for cell in table.columns[0].cells))]                
                    
        if isVertical == False: 
            if len(text) > 0:            
                try:
                    va = text[1]
                    gtext.append(va)
                except:
                    continue
        else: 
            row_data = dict(zip(keys, text)) 
            data.append(row_data)
     
    if isVertical == False:        
        row_data = dict(zip(keys, gtext)) 
        data.append(row_data) 
        
    df = pd.DataFrame(data)
    df = df.fillna('')
    return df

def GetHeadingLevel(stylename):
    name = stylename
    
    try:
        
        level = [int(s) for s in name.split() if s.isdigit()][0]
    
        return level
    except:
        return 1

   

root = None
parent = root

def CreateHeading(block):
    global parent
    item = Item()
    item.Content = block.text.strip()
    item.Type = block.style.name   
    
    parentLevel = GetHeadingLevel(parent.Type) if parent.Type != 'root' else 0   
    itemLevel = GetHeadingLevel(item.Type)
    
    
    # If the heading is new then add it to the root level
    if (GetHeadingLevel(item.Type) == 1): 
        root.Items.append(item) 
        item.Parent = root#item
        parent = item
    
    # Check if the new heading is a subheading of its parent or not
    elif ((parentLevel + 1) == itemLevel):
        parent.Items.append(item)
        item.Parent = parent
    
    # Get the last heading from the parent and then make that heading the parent of this newly created heading
    elif ((parentLevel + 1) < itemLevel):         
       
        parent = GetLastHeading()                          
        parent.Items.append(item)
        item.Parent = parent
        
    # Find the relivent parent and then add the new heading to its parent heading
    elif ((parentLevel + 1) > itemLevel):         
         
        parent = FindParent(parent, item)
        parent.Items.append(item)
        item.Parent = parent

        
    return item
    

def FindParent(par, item):
    parentLevel = GetHeadingLevel(par.Type)
    itemLevel = GetHeadingLevel(item.Type)    
   
    if((parentLevel + 1) > itemLevel):
        return FindParent(par.Parent, item)
    else:
        return par

    
def GetLastHeading():
     for i in reversed(parent.Items):
            if 'Heading' in i.Type:
                return i
                
def AddParagraph(head, block):
    item = Item()
    item.Type = block.style.name
    item.Content = block.text.strip()
    item.Parent = head
    head.Items.append(item)   
   
    pass

def AddTable(head, block):
    item = Item()
    item.Type = 'Table'
    item.Parent = head    
    item.Content = ParseTableToDataFrame(block)
    head.Items.append(item)
    pass

def GetWordDocTree(filename):
    global root
    global parent
    
    doc = Document(filename)
    root = Item()
    root.Type = "root"
    root.Content = filename
    parent = root
    
    head = root    
    for block in iter_block_items(doc):

        if ('Heading' in block.style.name) and len(block.text.strip()) > 0: #Heading
            
            head = CreateHeading(block)            
            pass
            
        
        if isinstance(block, Table): #Table             
            AddTable(head, block)
            pass
            
        
        elif len(block.text.strip()) > 0: # Paragraph and everything
            AddParagraph(head, block)
            pass
            
    root.Content = filename       
    return root        

    #return {"root": root, "filename": filename}    
    
        
        
        